# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartSLAB
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from builtins import range
import os

from PyQt5 import QtGui, uic
from PyQt5.QtWidgets import (QHBoxLayout,
                         QPushButton,
                         QDialog)
from PyQt5.QtGui import (QPixmap,
                         QIcon)

import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar

from qgis.core import QgsGeometry

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'chart.ui'))


class CtrlCraftOfficeShipmentsChart(QDialog, FORM_CLASS):

    def __init__(self, data, canvas, max_slabs, parent=None):
        super(CtrlCraftOfficeShipmentsChart, self).__init__(parent)
        self.setupUi(self)
        self.data = data
        self.canvas = canvas
        self.max_slabs = max_slabs

    def init(self, deposit, panel, blast):
        self.__init_chart(deposit, panel, blast)
        self.__init_buttons()

    def onClick(self, event):
        index = event.ind
        slabs = self.currentIdSlabs[index[0]]

        for i in range(0, self.buttons.count()):
            self.buttons.itemAt(i).widget().setEnabled(False)

        for i in range(0, len(slabs)):
            self.buttons.itemAt(i).widget().setEnabled(True)
            wkt = self.data.slab_wkt(slabs[i])
            self.buttons.itemAt(i).widget().setProperty('wkt', wkt)

    def zoomOnSlab(self, event):
        sender = self.sender()

        wkt = sender.property('wkt')
        geom = QgsGeometry.fromWkt(wkt)

        self.canvas.setExtent(geom.boundingBox())
        self.canvas.zoomOut()

    def __init_chart(self, deposit, panel, blast):
        # porticData = self.data.portic_data()
        shipments = self.data.shipments(deposit, panel, blast)

        # Match the data
        dataChartSlab = []
        dataChartSlabProbe = []
        dataChartPortic = []
        self.currentIdSlabs = []

        gateids = []
        for shipment in shipments:
            truck = shipment[4]
            unload_time = shipment[5]

            if not unload_time:
                continue

            # do not use seconds because the portic datetime is in minutes
            # only
            times = unload_time.split(':')
            unload_time = '{}:{}'.format(times[0], times[1])

            data = self.data.gate_data_entry(deposit, panel, blast,
                                             truck, unload_time)

            gate_radiometry = 0.0
            if data:
                gateid = data[0]
                if gateid not in gateids:
                    gateids.append(gateid)
                    gate_radiometry = data[1]

            valSlab = float(shipment[6]) if shipment[6] else 0.0
            valSlabProbe = float(shipment[7]) if shipment[7] else 0.0
            valSlabPortic = gate_radiometry
            valIdSlabs = shipment[8]

            dataChartSlab.append(valSlab)
            dataChartSlabProbe.append(valSlabProbe)
            dataChartPortic.append(valSlabPortic)
            self.currentIdSlabs.append(valIdSlabs)

        # Build chart
        fig, ax = plt.subplots()
        ax.scatter(dataChartPortic, dataChartSlabProbe, c='b', s=30, alpha=0.5, picker=5)
        ax.scatter(dataChartPortic, dataChartSlab, c='r', s=30, alpha=0.5, picker=5)

        ax.set_xlabel(self.tr('Gate grade'))
        ax.set_ylabel(self.tr('Slabs grade (estimated: red, probe: blue'))
        ax.set_title(self.tr('Correlation between probe and gate'))

        ax.grid(True)
        fig.tight_layout()

        chartCanvas = FigureCanvas(fig)
        chartCanvas.mpl_connect('pick_event', self.onClick)

        toolbar = NavigationToolbar(chartCanvas, self)

        self.vLayout.addWidget(toolbar)
        self.vLayout.addWidget(chartCanvas)

    def __init_buttons(self):
        self.buttons = QHBoxLayout()
        iconName = 'iconSlab_small.png'
        path = os.path.join(os.path.dirname(__file__), 'img', iconName)
        pix = QPixmap(path)
        icon = QIcon(pix)

        for i in range(0, self.max_slabs):
            button = QPushButton('Slab {}'.format(i+1))
            button.setIcon(icon)
            button.setEnabled(False)
            button.setProperty('slab', i)
            button.clicked.connect(self.zoomOnSlab)
            self.buttons.addWidget(button)

        self.vLayout.addLayout(self.buttons)
